// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.io.*;
import java.net.*;
import java.util.Queue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;


public class CommClient extends Thread{

    private final static int  READTIMEOUT=150;
    private final static long RESPTIMEOUT=5;
    private final static int SLEEPTIME=300;
    private final static int CAPACITY=10;
    private String ip=null;
    private int port=1024;
    private Socket sock=null;
    private volatile boolean connected=false;
    private MsgProcessor msgproc=null;
    private BlockingQueue<byte[]> responses=null;
    private BlockingQueue<Thread> notifs=null;
    private MsgSubscriber asksubscr=null;
    private MsgSubscriber infosubscr=null;


    public CommClient(){
        super("CommClient");
        responses=new LinkedBlockingQueue<byte[]>(CAPACITY);
        notifs=new LinkedBlockingQueue<Thread>(CAPACITY);
    }
    
    public void setServerAddress(String serverip, int serverport)
                                                            throws IOException{
        ip=serverip;
        port=serverport;
        
        InetAddress addr=InetAddress.getByName(ip);
        sock=new Socket(addr, port);
        sock.setSoTimeout(READTIMEOUT);
              
        msgproc=new MsgProcessor(sock.getInputStream(), sock.getOutputStream());
        connected=true;
        start();
    }

    public void sendMsg(byte[] msg) throws IOException{
        msgproc.writeMsg(msg);
    }

    public void sendMsg(byte[] msg, int offset, int len) throws IOException{
        msgproc.writeMsg(msg, offset, len);
    }

    public byte[] getResponse() throws IOException{
        byte[] aux=null;
        try{
            aux=responses.poll(RESPTIMEOUT, TimeUnit.SECONDS);
            if(aux==null)
                throw new IOException("Server response time out expired");

        }catch(InterruptedException ie){
            throw new IOException("Interrupted operation");
        }
        
        return aux;
    }

    public void setSubscriber(int scrtype, MsgSubscriber scbr){

        if(scrtype==MsgSubscriber.ASK_SUBSCRIBER)
            asksubscr=scbr;
        else if(scrtype==MsgSubscriber.INFO_SUBSCRIBER)
            infosubscr=scbr;
    }

 
    private int getFuncId(byte[] msg){
        return ((int)msg[1]) & 0xFF;
    }

    @Override
    public void run(){
        Notifier notifier=null;
        int source=0;
        String errmsg=null;
        byte[] msg=null;

        if(!connected){
            errmsg="Unable to connect with server";
            throw new IllegalStateException(errmsg);
        }else{

            while(connected)
                try{
                    msg=msgproc.readMsg();                                        
                    if(msg==null || msg.length==0){
                        connected=false;
                    }else if(msg[0]==Msg.RESP){
                        responses.add(msg);                        
                    }else if(msg[0]==Msg.ASK){
                        source=getFuncId(msg);
                        if(asksubscr!=null && asksubscr.getSource()==source){
                            notifier=new Notifier(asksubscr, msg, notifs);
                            notifs.offer(notifier);
                            notifier.start();
                        }
                    }else if(msg[0]==Msg.INFO){
                        source=getFuncId(msg);
                        if(infosubscr!=null && infosubscr.getSource()==source){
                            notifier=new Notifier(infosubscr, msg, notifs);
                            notifs.offer(notifier);
                            notifier.start();
                        }
                    }                    
                }catch(SocketTimeoutException e){
                    try{
                        sleep(SLEEPTIME);
                        }catch(InterruptedException ie){}
                }catch(IOException io){
                    connected=false;
                    io.printStackTrace(System.err);
                }

            System.err.println("Connection with server is broken");
        }
    }

    public void release() throws IOException{

        if(connected)       
            try{
                connected=false;
                this.join();
                msgproc.release();
            }catch(InterruptedException ie){}

        if(sock!=null)
            sock.close();        
    }
}


class Notifier extends Thread{

    private MsgSubscriber subscr=null;
    private byte[] msg=null;
    private Queue<Thread> notifs=null;

    public Notifier(MsgSubscriber subscr, byte[] msg, Queue<Thread> notifs){
        super("Notifier");
        this.subscr=subscr;
        this.msg=msg;
        this.notifs=notifs;
    }

    @Override
    public void run(){
        Thread thread=notifs.peek();
        while(thread!=null && thread!=this)
            try{
                thread.join();
            }catch(InterruptedException ie){
                ie.printStackTrace(System.err);
            }finally{
                thread=notifs.peek();
            }

        subscr.deliverMsg(msg);
        if(thread==this)
            notifs.poll();
    }
}
