// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.ByteBuffer;
import java.util.*;


public class MsgOptParser {

    private static HashMap<String, Class> paramstype;
    private static ArrayList<Class> supptypes;
    private final static int NUMINFOITEMS=5;
    private final static int INTCLS=0;
    private final static int STRCLS=1;
    private final static int BLNCLS=2;
    private final static String csname="US-ASCII";
    private final static String macformat="%02x:%02x:%02x:%02x:%02x:%02x";
    private static Charset ascii=null;

    public static void init() throws UnsupportedEncodingException{
        paramstype=new HashMap<String,Class>(8);
        paramstype.put(Msg.STAGE, int.class);
        paramstype.put(Msg.WIFACE, String.class);
        paramstype.put(Msg.FCS, boolean.class);
        paramstype.put(Msg.MONHDR, int.class);
        paramstype.put(Msg.SUBNET, String.class);
        paramstype.put(Msg.SEARCH, int.class);
        paramstype.put(Msg.MINRET, int.class);
        paramstype.put(Msg.MINTMO, int.class);

        supptypes=new ArrayList<Class>(3);
        supptypes.add(int.class);
        supptypes.add(String.class);
        supptypes.add(boolean.class);

        ascii=Charset.forName("US-ASCII");
    }

    public static ByteBuffer getBytes(String message){
        return ascii.encode(message);
    }

    public static String getString(byte[] msg, int offset, int len){
        String str=null;

        try{
            str=new String(msg, offset, len, csname);
        }catch(UnsupportedEncodingException ue){
            ue.printStackTrace(System.err);
        }
        return str;
    }

    public static byte[] getRawMAC(String mac){
        byte[] rawmac=null;
        boolean parserror=false;

        if(mac!=null || mac.length()>10){
            String[] macbytes=mac.split(":");
            if(macbytes.length==6){
                int index=0;
                int val=0;
                try{
                    rawmac=new byte[6];
                    for(String mb: macbytes){
                        val=Integer.parseInt(mb, 16);
                        rawmac[index++]=(byte)val;
                    }
                }catch(NumberFormatException nfe){
                    parserror=true;
                }

                if(!parserror)
                    return rawmac;
            }
        }
        return null;
    }

    public static String[] parseInterfaceList(byte[] msg, int offset, int len){
        int index=offset, itemindex=0;
        ArrayList<String> lst=new ArrayList<String>();
        String ifaces[]=null, tmp=null;
        try{
            itemindex=indexOf(msg, Msg.ITEM_SEP, offset);
            while(itemindex>0 && itemindex<len+offset){
                tmp=new String(msg, index, itemindex-index, csname);
                lst.add(tmp);
                index=itemindex+1;
                itemindex=indexOf(msg, Msg.ITEM_SEP, index);
            }

            ifaces=new String[lst.size()];
            lst.toArray(ifaces);

        }catch(UnsupportedEncodingException ue){
            ue.printStackTrace(System.err);
        }
        
        return ifaces;
    }

    public static int nextItemSepPos(byte msg[], int offset, int stoposition){
        int ind=offset;
        
        while(ind<stoposition && msg[ind]!=Msg.ITEM_SEP) ind++;

        return ind;
    }

    public static Map.Entry<String, String> nextItem(byte msg[], int offset){
        int indval=offset, inditem=-1;
        String name=null, value=null;
        
        while(indval<msg.length && msg[indval]!=Msg.VALUE_SEP) indval++;

        if(indval<msg.length){
            inditem=indval+1;
            while(inditem<msg.length && msg[inditem]!=Msg.ITEM_SEP)
                inditem++;
        }

        if(inditem>0 && inditem<msg.length)
            try{
                name=new String(msg, offset, indval-offset, csname);
                value=new String(msg, indval+1, inditem-indval-1, csname);
                return new OptParam(name, value);
            }catch(UnsupportedEncodingException ue){
                ue.printStackTrace(System.err);
            }
        
        return null;
    }
    
    public static HashMap parseConfigOpts(byte msg[], int offset, int len){

        HashMap<String, Object> opts=new HashMap<String, Object>();
        int index=0, paramindex=0, itemindex=0, clsindex=0;
        int toplim=0, intval=0, tmplen=0;
        Object value=null;
        String param=null;
        Class cls=null;

        index=offset;
        paramindex=indexOf(msg, Msg.VALUE_SEP, offset);
        itemindex=indexOf(msg, Msg.ITEM_SEP, offset);

        while(itemindex>0 && itemindex<len+offset)
            try{
                param=new String(msg, index, paramindex-index, csname);
                cls=paramstype.get(param);
                if(cls!=null && (clsindex=supptypes.indexOf(cls))>=0){
                    value=null;
                    switch(clsindex){                        
                        case INTCLS:
                            toplim=Math.min(3, itemindex-paramindex-1);
                            toplim+=paramindex;
                            intval=0;
                            for(int i=paramindex+1; i<=toplim; i++)
                                intval=intval*256 + ((int)msg[i] & 0xFF);

                            value=new Integer(intval);
                            break;

                        case STRCLS:
                            tmplen=itemindex-paramindex-1;
                            value=new String(msg, paramindex+1, tmplen, csname);
                            break;

                        case BLNCLS:
                            value=new Boolean(msg[paramindex+1]!=0);
                            break;
                    }

                    if(value!=null)
                        opts.put(param, value);
                }

                index=itemindex+1;
                paramindex=indexOf(msg, Msg.VALUE_SEP, index);
                if(paramindex>0)
                    itemindex=indexOf(msg, Msg.ITEM_SEP, paramindex+1);
                else
                    itemindex=-1;

            }catch(UnsupportedEncodingException uee){
                throw new RuntimeException(uee);
            }

            return opts;
    }

    public static List<BSSInfo> parseScanInfo(byte[] msg, int offset, int len){
        List<BSSInfo> info=new LinkedList<BSSInfo>();
        BSSInfo bssinfo=null;
        String essid=null, bssid=null;
        int index=offset, end=offset+len, tmp=0;
        int item=0, lenessid=0;

        while(index < end){
            switch(item){
                case 0:
                    lenessid=(int)msg[index];
                    break;
                    
                case 1:
                    try{
                        if(lenessid>0){
                            essid=new String(msg, index, lenessid, csname);
                            index+=lenessid-1;
                        }else{
                            essid=null;
                            index-=1;
                        }
                        break;
                    }catch(UnsupportedEncodingException ue){
                        throw new RuntimeException(ue);
                    }

                case 2:
                    bssid=String.format(macformat, msg[index],   msg[index+1],
                                                   msg[index+2], msg[index+3],
                                                   msg[index+4], msg[index+5]);
                    index+=5;
                    bssinfo=new BSSInfo();
                    bssinfo.setEssid(essid);
                    bssinfo.setBssid(bssid);
                    info.add(bssinfo);
                    break;

                case 3:
                    bssinfo.setChannel((int)msg[index]);
                    break;

                case 4:
                    if(msg[index]<0)
                        tmp=128+(int)msg[index];
                    else
                        tmp=(int)msg[index]-128;

                    bssinfo.setSnr(tmp);
                    break;

                case 5:
                    bssinfo.setQuality((int)msg[index]);
                    break;
            }
            
            index++;
            if(item >= NUMINFOITEMS)
                item=0;
            else
                item++;            
        }

        return info;
    }

    public static String parseESSID(byte[] msg, int offset, int len){
        String essid=null;

        try{
            essid=new String(msg, offset, len, csname);
        }catch(UnsupportedEncodingException ue){
            throw new RuntimeException(ue);
        }

        return essid;
    }

    
    public static String reprBytes(byte[] bytearray){
        StringBuffer sb=new StringBuffer("[ ");
        for(byte b : bytearray){
            sb.append(Integer.toHexString(((int)b) & 0xFF));
            sb.append(", ");
        }
        int len=sb.length();
        sb.delete(len-2, len);
        sb.append(" ]");
        return sb.toString();
    }

    private static int indexOf(byte[] data, byte val, int offset){
        int index=(offset<0)?0:offset;

        while(index<data.length && data[index]!=val) index++;

        if(index>=data.length)
            index=-1;

        return index;
    }
}

class OptParam implements Map.Entry<String,String>{
    
    private String name=null;
    private String val=null;

    public OptParam(String name, String val){
        this.name=name;
        this.val=val;
    }

    public String getKey(){
        return name;
    }

    public String getValue(){
        return val;
    }

    public String setValue(String value){
        String tmp=this.val;
        this.val=value;
        return tmp;
    }

    @Override
    public boolean equals(Object obj){
        if(obj instanceof Map.Entry){
            Object objkey=((Map.Entry)obj).getKey();
            return name.equals(objkey);
        }
        return false;
    }

    @Override
    public int hashCode(){
        return name.hashCode();
    }
}