# -*- coding: UTF-8 -*-
# Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import array

import msg
import actions
from iwlibs import getNICnames
from iwlibs import Wireless


class ServerConf(object):   

    def __init__(self):
        self.stage=0
        self.subnet=None
        self.search_type=None
        self.if_MAC=None
        self.if_mon=None
        self.if_cli=None
        self.has_fcs=None
        self.ap_MAC=None        
        self.ssid=None
        self.mon_head_type=None
        self.mon_head=None
        self.srt_timeout=None
        self.mdm_timeout=None
        self.lng_timeout=None
        self.min_retries=None
        self.max_retries=None

    def detach_bssid(self):
        self.ssid=None
        self.ap_MAC=None

    def __get_attr_set_cli(self, advstage=False):        
        ADVANCE=128
        return (
                (advstage and (self.stage+ADVANCE)) or self.stage, self.if_cli,
                self.has_fcs,     self.mon_head_type,
                self.subnet,      self.search_type,
                self.min_retries, self.srt_timeout)
                      

    def is_updated_by_client(self):        
        allsetted=True

        for attr in self.__get_attr_set_cli():
            if attr is None:
                allsetted=False
                break

        return allsetted

    def get_cli_config_str(self, advstage=False):        
        result=None

        if self.is_updated_by_client():
            params=(msg.STAGE,  msg.WIFACE,
                    msg.FCS,    msg.MONHDR,
                    msg.SUBNET, msg.SEARCH,
                    msg.MINRET, msg.MINTMO)

            values=self.__get_attr_set_cli(advstage)
            tostring=lambda x: (isinstance(x, int) and chr(x)) or str(x)
            pairs=zip(params, map(tostring ,values))
            result=array.array("c", "")

            for pair in pairs:
                result.extend(pair[0])
                result.extend(msg.VALUE_SEP)
                result.extend(pair[1])
                result.extend(msg.ITEM_SEP)

            result=result.tostring()
        
        return result

    def get_stage(self):        
        return self.stage

    def set_stage(self, _stage):
        self.stage=_stage

    def get_subnet(self):        
        return self.subnet

    def set_subnet(self, subnetprefix):
        self.subnet=subnetprefix

    def get_search_type(self):        
        return self.search_type

    def set_search_type(self, searchid):
        self.search_type=searchid

    def get_if_MAC(self):        
        return self.if_MAC

    def set_if_MAC(self, mac):
        self.if_MAC=mac

    def get_if_mon(self):        
        return self.if_mon

    def set_if_mon(self, ifname):
        self.if_mon=ifname

    def get_if_cli(self):        
        return self.if_cli

    def set_if_cli(self, ifname):
        self.if_cli=ifname

    def get_has_fcs(self):        
        return self.has_fcs

    def set_has_fcs(self, hasfcs):
        self.has_fcs=hasfcs

    def get_ap_MAC(self):       
        return self.ap_MAC

    def set_ap_MAC(self, mac):
        self.ap_MAC=mac

    def get_SSID(self):        
        return self.ssid

    def set_SSID(self, _ssid):
        self.ssid=_ssid

    def get_mon_head_type(self):        
        return self.mon_head_type

    def set_mon_head_type(self, monheadid):
        self.mon_head_type=monheadid
        if(monheadid==0):
            self.set_mon_head(None)
        elif(monheadid==1):
            self.set_mon_head(actions.build_rt_basic())
        

    def get_mon_head(self):        
        return self.mon_head

    def set_mon_head(self, header):        
        self.mon_head=header

    def get_monitor_packet(self):        
        head=self.mon_head
        
        if(head is None):
            return Dot11
        else:
            return type(head)
        
    def get_short_timeout(self):
        return self.srt_timeout

    def set_short_timeout(self, timeout):
        self.srt_timeout=timeout

    def get_medium_timeout(self):
        return self.mdm_timeout

    def set_medium_timeout(self, timeout):
        self.mdm_timeout=timeout

    def get_long_timeout(self):
        return self.lng_timeout

    def set_long_timeout(self, timeout):
        self.lng_timeout=timeout

    def get_min_retries(self):
        return self.min_retries

    def set_min_retries(self, min_ret):
        self.min_retries=min_ret

    def get_max_retries(self):
        return self.max_retries

    def set_max_retries(self, max_ret):
        self.max_retries=max_ret




class SessionConf:
    
    def __init__(self):
        self.capframe=None
        self.keystream=None
        self.injframe=None

    def get_WEP_frame(self):    
        return self.capframe

    def set_WEP_frame(self, _frame):
        self.capframe=_frame

    def get_inject_ARP(self):
        return self.injframe

    def set_inject_ARP(self, frame):
        self.injframe=frame

    def get_keystream(self):        
        return self.keystream
    
    def set_keystream(self, kstr):
        self.keystream=kstr



class IfaceList(object):    
    ETHER_LINK_TYPE=0x01
    MONITOR_LINK_TYPES=(  0x69, 
                          0x7F, 
                          0x321,
                          0x322,
                          0x323
                         )
                    
    def __init__(self):
        self.mng=[]
        self.mon=[]
        self.macs={}

    def reset(self):
        self.mng=[]
        self.mon=[]
        self.macs={}

    def is_mon_lt_supported(self, lt):
        return lt in self.MONITOR_LINK_TYPES

    def add_iface(self, ifname, macaddr, type):
        tuple=(ifname, macaddr)
        iflist=None

        if(type=='Managed'):
            iflist=self.mng
        elif(type=="Monitor"):
            iflist=self.mon

        if len(iflist)==0:
            iflist.append(tuple)
        else:
            i=0
            notfound=True
            thisname=None

            while(i < len(iflist) and notfound):
                thisname=iflist[i][0]
                if(ifname < thisname):
                    iflist.insert(i, tuple)
                    notfound=False
                elif(ifname > thisname):
                    i=i+1
                else:
                    return

            if(notfound):
                iflist.append(tuple)

                

        if(self.macs.has_key(macaddr)):
            iflist=self.macs[macaddr]
            if(type=="Managed"):
                iflist.insert(0, ifname)
            elif(type=="Monitor"):
                iflist.append(ifname)
        else:
            self.macs[macaddr]=[ifname]
            
    def list_ifaces(self, type):
        
        iflist=None
        result=[]

        if(type=='Managed'):
            iflist=self.mng
        elif (type=="Monitor"):
            iflist=self.mon

        for tuple in iflist:
            result.append(tuple[0])

        return result

    def get_iface_by_address(self, macaddr, type, count=1):

        if(self.macs.has_key(macaddr)):
            iflist=self.macs[macaddr]
            if type=="Managed":
                return iflist[0]
            elif type=="Monitor":
                lng=len(iflist)
                if(count>0 and count<lng):
                    return iflist[count]
                else:
                    return None
        else:
            return None
        

    def update(self):
        wiface=None
        mode=None
        linktype=None
        hwaddr=None
        iflist=getNICnames()

        self.reset()
        for ifname in iflist:
            wiface=Wireless(ifname)
            mode=wiface.getMode()

            if mode=="Managed":
                linktype, hwaddr=wiface.getHWAddress()
                if linktype==self.ETHER_LINK_TYPE:
                    self.add_iface(ifname, hwaddr, "Managed")
            elif mode=="Monitor":
                linktype, hwaddr=wiface.getHWAddress()
                if self.is_mon_lt_supported(linktype):
                    self.add_iface(ifname, hwaddr, "Monitor")
            else:
                pass

            wiface.release()


srvconfig=ServerConf()
stagedata=SessionConf()
iflist=IfaceList()
cachescan=[]
