# -*- coding: UTF-8 -*-
# Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#


from scapy.packet import *
from scapy.layers.dot11 import *
from scapy.config import conf
from scapy.data import ETH_P_ALL
from scapy.arch import X86_64
from scapy.arch.linux import SOL_SOCKET, SO_ATTACH_FILTER, _flush_fd
from scapy.error import log_runtime
import socket
import os
import struct
import binascii
from time import time, sleep
from select import select
from threading import Thread, Event
from filters import *

def attach_filter(s, iface=conf.iface, filter=''):

    try:
        f = os.popen("%s -i %s -ddd -s 0 '%s'" % (conf.prog.tcpdump, iface, filter))
    except OSError, msg:
        log_interactive.warning("Failed to execute tcpdump: (%s)"%(msg,))
        return

    lines = f.readlines()
    if f.close():
        raise Scapy_Exception("Filter parse error")

    nb = int(lines[0])
    bpf = ""
    for l in lines[1:]:
        bpf += struct.pack("HBBI", *map(long, l.split()))
    
    if X86_64:
        bpfh = struct.pack("HL", nb, id(bpf) + 36)
    else:
        bpfh = struct.pack("HI", nb, id(bpf) + 20)

    s.setsockopt(SOL_SOCKET, SO_ATTACH_FILTER, bpfh)


class Dot11FXSession(object):    
    MSG_READY="READYTOSEND\n"
    MSG_SENT ="ENDSENDING\n"
    MSG_END  ="ENDRECEPTION\n"
    CONTINUE_COND=0
    RECEIVE_READY_COND=1
    END_OF_RECPTN_COND=2
    __sessioncounter=0

    def __init__(self, _iface, _ifpacket, _sockfilter):
        
        try:
            Dot11FXSession.__sessioncounter+=1
            self.id=Dot11FXSession.__sessioncounter
            self.frames=[]
            self.iface=_iface
            self.ifpacket=_ifpacket
            
            self.rdpipe,self.wrpipe=os.pipe()

            self.conditions=[]
            evt=None
            for i in range(3):
                evt=Event()
                evt.clear()
                self.conditions.append(evt)
            del i, evt

            self.sock=socket.socket(socket.AF_PACKET,
                                    socket.SOCK_RAW,
                                    socket.htons(ETH_P_ALL))

            self.sock.bind((self.iface, ETH_P_ALL))
            self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 2 ** 20)
            self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2**10)

            attach_filter(self.sock, self.iface, _sockfilter)
            _flush_fd(self.sock)

        except EnvironmentError:
            raise Exception, \
                "Session initialization error"
                

    def release(self):

        try:
            self.sock.close()
        except OSError:
            log_interactive.error(\
                "Error in module '%s', class '%s', function '%s':"\
                "\n\t%s"
                % ("sndrcvth", "Dot11FXSession", "release", \
                   "Error while releasing resources"))

    def get_session_id(self):
        return self.id

    def get_frames(self):
        return self.frames

    def clear_frames(self):
        self.frames[:]=[]

    def get_iface(self):
        return self.iface

    def get_packet_type(self):
        return self.ifpacket

    def get_socket(self):
        return self.sock

    def get_readpipe(self):
        return self.rdpipe

    def get_writepipe(self):
        return self.wrpipe

    
    @classmethod
    def cond_name(cls, ind):
        if ind==cls.CONTINUE_COND:
            return "CONTINUE"
        elif ind==cls.RECEIVE_READY_COND:
            return "RECEIVE_READY"
        elif ind==cls.END_OF_RECPTN_COND:
            return "END_OF_RECEPTION"

        return "UNKNOWN"
    

    def wait_condition(self, cond):
        self.conditions[cond].wait()

    def set_condition(self, cond):
        self.conditions[cond].set()

    def clear_condition(self, cond):
        self.conditions[cond].clear()


class ReceiverThread(Thread):
    MTU=2500
    
    def __init__(self, _session):        
        Thread.__init__(self)
        self.setName('Recv')
        sock=_session.get_socket()
        pipe=_session.get_readpipe()
        self.input=[pipe, sock]
        self.l2pkt=_session.get_packet_type()
        self.frames=_session.get_frames()
        self.session=_session
        self.ctrlchan=os.fdopen(pipe,'r')
        self.stop=False
       
        self.filters=None
        self.timeout=None

    def set_filters(self, _filters):
         self.filters=_filters

    def set_timeout(self, _timeout):
        self.timeout=_timeout

    def release(self):
        try:
            self.stop=True
            self.session.set_condition(Dot11FXSession.CONTINUE_COND)
            self.ctrlchan.close()
        except OSError:
            log_runtime.error(\
                "Error in module '%s', class '%s', function '%s':"\
                "\n\t%s"
                % ("sndrcvth", "ReceiverThread", "release", \
                   "Error while releasing resources"))

    
    def run(self):
        pipe=self.input[0]
        sock=self.input[1]


        while(not self.stop):
            recv_enable=False
            time_run=False
            
            remain_time=10.0
            elapsed=0.0
            timecheck=0.0
            now=0.0
            filtindex=0
            fdtuple,msj=None, None
            buf,sa_ll,pkt=None, None, None

            
            self.session.wait_condition(Dot11FXSession.CONTINUE_COND)
            if(self.stop):
                break
            else:            
                self.session.clear_condition(Dot11FXSession.CONTINUE_COND)
                
            numfilters=len(self.filters)
            while(filtindex<numfilters and remain_time>0.0):
                
                fdtuple=select(self.input,[],[],remain_time)
                if pipe in fdtuple[0]:
                    msj=self.ctrlchan.readline()                    
                    if msj==Dot11FXSession.MSG_READY:
                        recv_enable=True                        
                        _flush_fd(sock)
                        self.session.set_condition(\
                                        Dot11FXSession.RECEIVE_READY_COND)
                    elif msj==Dot11FXSession.MSG_SENT:
                        remain_time=self.timeout/1000.0
                        timecheck=time.time()
                        time_run=True
                    elif msj==Dot11FXSession.MSG_END:
                        self.stop=True
                        break
                elif sock in fdtuple[0]:
                    if recv_enable:
                        try:
                            buf,sa_ll=sock.recvfrom(MTU)
                            pkt=self.l2pkt(buf)

                            if(self.filters[filtindex](pkt)):
                                self.frames.append(pkt)
                                filtindex+=1
                            
                        except Exception, msg:
                            log_runtime.error(\
                            "Error while parsing packet: "+str(msg)+
                            "\n Data readed: "+binascii.hexlify(buf))
                            
                        if time_run:
                            now=time.time()
                            elapsed=now-timecheck
                            if elapsed<remain_time:
                                remain_time-=elapsed
                                timecheck=now
                            else:
                                remain_time=-1.0
                                recv_enable=False
                                
                    else:
                        buf=sock.recvfrom(MTU)
                      
                else:
                    remain_time=-1.0
                    recv_enable=False

            
            self.session.set_condition(Dot11FXSession.END_OF_RECPTN_COND)

            

class Sender(object):
   
    def __init__(self, _session):
   
        self.session=_session
        self.sock=_session.get_socket()
        self.ctrlchan=os.fdopen(_session.get_writepipe(),'w')
        self.minlngend=None
        self.endpkt=None
        self.num_endpkts=1
        self.rts=None
        
    def set_min_lng_end(self, _minlength):
        self.minlngend=_minlength

    def set_end_frame(self, _endpkt):
        self.endpkt=str(_endpkt)

    def set_num_end_frames(self, num):
        self.num_endpkts=num

    def set_rts(self, _rts):
        self.rts=str(_rts)   

    def send_dot11(self, pktls):
    
        for i in range(len(pktls)):
            item=pktls[i]
            if(isinstance(item, Packet)):
                pktls[i]=str(item)
            elif(isinstance(item, str)):
                pass
            elif(type(item) in (int,float)):
                pktls[i]=item*10**(-6)

        self.session.set_condition(Dot11FXSession.CONTINUE_COND)    
        self.session.clear_condition(Dot11FXSession.RECEIVE_READY_COND)
        self.ctrlchan.write(Dot11FXSession.MSG_READY)
        self.ctrlchan.flush()
        self.session.wait_condition(Dot11FXSession.RECEIVE_READY_COND)

        
        for pkt in pktls:
            if(isinstance(pkt,float)):
                sleep(pkt)
            elif(self.rts is None):                
                self.sock.send(pkt)
            else:
                self.sock.send(self.rts)
                sleep(1)
                self.sock.send(pkt)

        
        self.ctrlchan.write(Dot11FXSession.MSG_SENT)
        self.ctrlchan.flush()

        
        self.session.wait_condition(Dot11FXSession.END_OF_RECPTN_COND)

        frames=self.session.get_frames()
        if(self.minlngend is not None and len(frames)>=self.minlngend):
            for i in range(self.num_endpkts):
                self.sock.send(self.endpkt)

        frmcp=[]
        for frame in frames:
            frmcp.append(frame)

        self.session.clear_frames()

        self.session.clear_condition(Dot11FXSession.END_OF_RECPTN_COND)
        return frmcp

    def release(self):
        try:
            self.ctrlchan.close()
        except OSError:
            log_interactive.error(\
                "Error in module '%s', class '%s', function '%s':"\
                "\n\t%s"
                % ("sndrcvth", "Sender", "release", \
                   "Error while releasing pipe"))


class SendReceiveDot11(object):

    def __init__(self, _iface, _ifpacket, _sockfilter):
        self.session=Dot11FXSession(_iface, _ifpacket, _sockfilter)
        self.sender=Sender(self.session)
        self.receiver=ReceiverThread(self.session)
        self.receiver.set_filters([lambda x: False])
        self.receiver.set_timeout(5)
        self.receiver.start()
        self.retries=1
        self.minlength=1

    def set_filters(self, _filters):
        self.receiver.set_filters(_filters)

    def set_timeout(self, _timeout):
        self.receiver.set_timeout(_timeout)

    def set_retries(self, _num_retries):
        self.retries=_num_retries

    def set_min_wanted_frames(self, _numframes):
        self.minlength=_numframes

    def set_end_frame(self, _endframe):
        self.sender.set_end_frame(_endframe)

    def set_end_frame_threshold(self, _numframes):
        self.sender.set_min_lng_end(_numframes)

    def set_num_end_frames(self, num):
        self.sender.set_num_end_frames(num)

    def set_rts(self, _rtsframe):
        self.sender.set_rts(_rtsframe)
    
    def send_fa(self, pkt, filters, timeout=5, _retries=1, _minlength=1,
                _rts=None, _minlngend=None, _endpkt=None, _num_endpkts=None):

        self.receiver.set_filters(filters)
        self.receiver.set_timeout(timeout)
        self.retries=_retries
        self.minlength=_minlength
        
        if(_rts is not None):
            self.sender.set_rts(_rts)

        if(_minlngend is not None):
            self.sender.set_min_lng_end(_minlngend)

        if(_endpkt is not None):
            self.sender.set_end_frame(_endpkt)

        if(_num_endpkts is not None):
            self.sender.set_num_end_frames(_num_endpkts)
        
        return self.send(pkt)


    def send(self, pkt):
 
        nrets=self.retries+1
        nsent=1
        min=self.minlength
        frames=[]
        pklst=[pkt]

        while nsent<nrets and len(frames)<min:
            frames=self.sender.send_dot11(nsent*pklst)
            nsent+=1

        return frames

    def release(self):
 
        self.sender.release()
        self.receiver.release()
        self.session.release()
